/* IRCfs - IRC FileServ for *nix.
 * Copyright (C) 2002  Nick 'Zaf' Clifford <zaf@nrc.co.nz>
 * For licensing details, refer to the LICENSE file in the source 
 * code directory.
 */
#ifndef __include_ipfunc_h__
#define __include_ipfunc_h__

struct in_addr;

struct dotquote {
	unsigned char a;
	unsigned char b;
	unsigned char c;
	unsigned char d;
};

struct ipv4_addr {
	/* Wether ulong is the address in Network Byte Order
	 * depends on the endianness of your machine, so please
	 * use it only for copying two ipv4_addr arrays, and use
	 * ipv4_to_in_addr to convert to a long in Network Byte
	 * Order.
	 */
	union ipv4_addr_u {
		struct dotquote sep;
		unsigned long ulong;
		unsigned char arr[4];
	} addr;
};
struct ipv4_range {
	struct ipv4_addr ip;
	struct ipv4_addr nm;
};


/* Converts a string in dotted quotted form to an ipv4_addr structure.
 * The following formats are accepted:
 * a.b.c.d
 * a.b.c.d/bitnum
 * a.b.c.d/e.f.g.h
 *
 * The bitnum field corisponds to the number of 1 bits in the netmask.
 * eg /24 means 24 bits are 1, which works out to 255.255.255.0
 */
int str_to_ipv4(const char *str, struct ipv4_range *ipv4);

/* Does the same as above, except expects only an IPv4 address.
 */
int str_to_ipv4_addr(const char *str, struct ipv4_addr *ipv4_a);

/* The following converts an ipv4 structure to a string.
 * The output depends on the style parameter and the netmask setting.
 * of the ipv4 address
 *
 * Size of buffer: 
 * If the output *could* contain a netmask, then a buffer must
 * be atleast 32 bytes big (to include the NULL character)
 * If the output only contains an IP address, then a buffer must
 * be 16 bytes big.
 *
 * Style settings:
 * IPV4_IP_ONLY
 *    Output is always in the form:
 *    a.b.c.d
 * 
 * IPV4_IP_OPT_FULL_NM
 *    If netmask is anything but all 1's (eg /255.255.255.255) then
 *    the output is in the form:
 *    a.b.c.d/e.f.g.h
 *    otherwise its simply: a.b.c.d
 *    
 * IPV4_IP_OPT_SMART_NM
 *    If the netmask is anything but all 1's (eg /255.255.255.255) then
 *    and the netmask is bit divisible, the output is in the form:
 *    a.b.c.d/bitnum
 *    If the netmask is wierd (eg: /123.211.12.1) then the output is
 *    in the form:
 *    a.b.c.d/e.f.g.h
 *    Otherwise, the output is in the form:
 *    a.b.c.d
 *    
 * IPV4_IP_WITH_SMART_NM   
 *    If the netmask is bit divisible, then the output is in the form:
 *    a.b.c.d/bitnum
 *    otherwise its:
 *    a.b.c.d/e.f.g.h
 *
 * IPV4_IP_WITH_FULL_NM
 *    The output is in the form:
 *    a.b.c.d/e.f.g.h
 * 
 * IPV4_FULL_NM_ONLY
 *    The output is in the form:
 *    e.f.g.h
 *
 * IPV4_SMART_NM_ONLY
 *    If the netmask is bit divisible, then the output is in the form:
 *    bitnum
 *    Otherwise its in the form:
 *    e.f.g.h
 */
int ipv4_to_str(struct ipv4_range *ipv4, char *buffer, 
		size_t bufflen, int style);

/* The following is the same as ipv4_to_str except it takes an
 * ipv4_addr struct instead. There is no style attribute since
 * it doesn't make any sense with only an address 
 */
int ipv4_addr_to_str(struct ipv4_addr *ipv4, char *buffer, 
		size_t bufflen);


/* Returns 0 if the ip range matches a loopback address
 * * eg a ipv4_comp() call will match this address to something in the
 * 127.0.0.0 region. (may not match 127.0.0.1 if for example, the
 * ipv4 struct was 127.0.0.5/32)
 * Returns -1 otherwise
 */
int ipv4_is_loopback(struct ipv4_range *ipv4);

/* Returns 0 if the ipv4 range is a wildcard (0.0.0.0/0.0.0.0)
 */
int ipv4_all_balls(struct ipv4_range *ipv4);

/* Returns 0 if the ipv4 address is a smurf (255.255.255.255)
 */
int ipv4_smurf(struct ipv4_addr *ipv4);

/* ipv4_comp(address,match)
 * This function compares an ipv4_addr address, and a ipv4_addr
 * address and its netmask.
 * Returns 0 on a match, and -1 otherwise.
 *
 * Examples: (Assume the strings are actually ipv4_addr structures)
 * ipv4_comp("1.2.3.4","0.0.0.0/0") = 0
 * ipv4_comp("1.2.3.4","255.255.255.255/32") = -1
 * ipv4_comp("1.2.3.4","255.255.255.255/0") = 0 (NOTE: the netmask!)
 * ipv4_comp("192.168.1.2","192.168.0.0/16") = 0
 * ipv4_comp("192.168.1.2","192.168.0.0/24") = -1 (NOTE: the netmask!)
 * ipv4_comp("192.168.1.10","192.168.1.0/255.255.255.248") = 0
 */
int ipv4_comp(struct ipv4_addr *address, struct ipv4_range *match);

/* Converts an in_addr to an ipv4_addr */
int inet_to_ipv4(struct in_addr *in, struct ipv4_addr *addr);

/* Checks if an IP address range has a legal netmask */
int ipv4_legal(struct ipv4_range *range);

enum ipv4_print_opts {
	IPV4_IP_ONLY,
	IPV4_IP_OPT_FULL_NM,
	IPV4_IP_OPT_SMART_NM,
	IPV4_IP_WITH_SMART_NM,   
	IPV4_IP_WITH_FULL_NM,
	IPV4_FULL_NM_ONLY,
	IPV4_SMART_NM_ONLY
};

/* Test function */
void test_ipv4(void);

#endif
